/*	Pop-up menu demo using FULL VT52 Library
 *
 *	Programmed by Erin Monaco
 *	 1994 DataBasement Software
 *	All Rights Reserved.
 *
 */

/*	#include Lattice headers
 *
 */

#include <aes.h>
#include <osbind.h>
#include <vdi.h>

/*	#include misc headers
 *
 */

#include "WTE_VT52.H"					/* Windowed VT52 Library Header */


/*	Global Variables, Definitions, Structs, etc
 *
 */

/*	Pop_Up States
 *
 */

#define	PU_NORMAL		0x00				/* normal entry */
#define	PU_DEFAULT	0x01				/* default entry - Return */
#define	PU_GRAYED		0x02				/* disabled entry */
#define	PU_RED			0x04				/* take note */

#define PU_SELECTED	0x80				/* entry selected - reversed */

#define ENTRIES			0x06				/* six pop-up entries */
#define PU_WIDTH		0x0C				/* maximum entry width */

#define PU_TSIZE		t10PT				/* entries text size */


int
	windh;					/* Pop-Up window's handle */

short
	states[6] =			/* Pop-Up entry states */
	{
		{PU_DEFAULT},
		{PU_GRAYED},
		{PU_NORMAL},
		{PU_RED},
		{PU_GRAYED},
		{PU_NORMAL}
	};

char
	*entries[6] =		/* Pop-Up pointers to entry texts */
	{
		{"  Default   "},
		{"  Disabled  "},
		{"  Normal    "},
		{"  Note      "},
		{"------------"},
		{"  Quit      "}
	};


/*	Function Prototypes
 *
 */

int		_main(void);		/* main prototype */
void	disp_popup(short numb_entries, char *entries[], short states[]);



/*	main
 *
 */

int _main(void)
{

	int
		item,						/* Menu item # */
		event;					/* event returned by evnt_multi() */

	short
		msg[8],					/* AES evnt_multi() message buffer */
		x,							/* mouse x position */
		y,							/* mouse y position */
		bstate,					/* final button state */
		kpress,					/* scancode of keypress */
		bclicks,				/* number of button clicks */

		row,						/* current row */
		col,						/* current column */
		w,							/* temporary width variable */
		h,							/* temporary height variable */
		d;							/* dummy variable */

	char
		PU_TITLE[] = " Menu ";


	/* initialize app with AES */
	appl_init();

	/* change mouse form to arrow */
	graf_mouse(ARROW, NULL);

	/* calculate work area needed for Pop-Up menu */
	wte_calc(PU_TSIZE, ENTRIES, PU_WIDTH, &w, &h);

	/* calculate total size of window needed */
	wind_calc(WC_BORDER, (NAME | MOVE), 1, 19, w, h, &d, &d, &w, &h);

	/* create Pop-Up's Window */
	windh = wind_create((NAME | MOVE), 1, 19, w, h);

	/* set Pop-Up's Title */
	wind_title(windh, PU_TITLE);

	/* open centered window */
	{

		short
			dw, dh;					/* used for desktops width and height */

		wind_get(DESK, WF_WXYWH, &d, &d, &dw, &dh);

		wind_open(windh, (dw - w)/2, (dh - h)/2, w, h);

	}	/* end open centered window block */

	/* initialize window for printing */
	wte_init(windh, &d, &d, &d);

	d = PU_TSIZE;

	/* insure text size set to correct points */
	wte_escf(windh, ESC_S, &d, &d);

	/* turn off line wrap */
	wte_escf(windh, ESC_w, &d, &d);

	/* lets initially display our pop-up menus text */

	/* We don't do anything because first thing that happens is we get
	 * a WM_REDRAW message and its done through there
	 */

	/* handle our Pop_Up Menu now */
	do
	{

		/* wait for keyboard and mouse clicks */
		event = evnt_multi
		(

			(MU_MESAG | MU_BUTTON | MU_KEYBD),
			1, 1, 1,
			0,0,0,0,0,
			0,0,0,0,0,
			msg,
			0,0,
			&x, &y, &bstate, &d, &kpress, &bclicks

		);

		/* if message, handle it */
		if(event & MU_MESAG)
		{

			/* can only MOVE the window, so check and handle */
			if(msg[0] == WM_MOVED)
			{

				GRECT DW;				/* desktops work area */


				/* get desktops work area */
				wind_get(DESK, WF_WXYWH, &DW.g_x, &DW.g_y, &DW.g_w, &DW.g_h);

				/* restrain window to desktops work area */
				rc_constrain(&DW, (GRECT *) &msg[4]);

				wind_set(msg[3], WF_CXYWH, msg[4], msg[5], msg[6], msg[7]);

				/* inform library of any change in position */
				wte_resize(windh, &d, &d);

			}	/* end if moved */

			if(msg[0] == WM_REDRAW)
			{

				/* just redraw menu!!! :) */
				disp_popup(ENTRIES, entries, states);

			}	/* end if redraw */

			if(msg[0] == WM_TOPPED)
			{

				wind_set(msg[3], WF_TOP, 0,0,0,0);

			}	/* end if topped */

		}	/* end if message event */

		/* if button, check if menu item clicked on */
		if(event & MU_BUTTON)
		{

			int
				status;					/* status */


			/* get row and column at mouse position at time of left-click */
			status = wte_fcor(windh, x, y, &row, &col);

			item = -1;	/* reset item # */

			/* if they clicked on a Pop-Up Menu item, handle it */
			if(status)
			{

				/* if not disabled, then select it */
				if(!(states[row] & PU_GRAYED))
				{

					int status;				/* status */


					/* clicked on a valid item, so row becomes the item # */
					item = row;

					/* turn on selected flag */
					states[item] = (short) (states[item] | PU_SELECTED);

					/* redisplay menu */
					disp_popup(ENTRIES, entries, states);

					/* lets be professional and wait for button to be released */
					evnt_button(0, 1, 0, &x, &y, &d, &d);

					/* what item was button released on */
					status = wte_fcor(windh, x, y, &row, &col);

					/* click on an item? */
					if(status)	/* yes */
					{

						/* is current row same as item # ? */
						if(row != item)	/* no */
						{

							/* aborted so deselect item and invalidate item # */
							states[item] = (short) (states[item] & ~PU_SELECTED);

							/* redisplay menu */
							disp_popup(ENTRIES, entries, states);

							item = -1;

						}	/* end if user aborted selection */

					}	/* end if released on an item */

					else		/* released outside window! */
					{

						/* aborted so deselect item and invalidate item # */
						states[item] = (short) (states[item] & ~PU_SELECTED);

						/* redisplay menu */
						disp_popup(ENTRIES, entries, states);

						item = -1;

					}	/* end else aborted outside menu */

				}	/* end if not disabled */

			}	/* end if clicked on menu item */

			/* handle item if selected */
			switch(item)
			{

				case 0:		/* item 0 */
				{

					form_alert(1, "[1][You have selected the |Default Item, by left |clicking it.][Cool]");
					break;

				}

				case 1:		/* item 1 */
				{

					form_alert(1, "[1][You have selected |a Disabled Item!][I Goofed]");
					break;

				}

				case 2:		/* item 2 */
				{

					form_alert(1, "[1][You have selected |a Normal Item.][Cool]");
					break;

				}

				case 3:		/* item 3 */
				{

					form_alert(1, "[1][Selected Red Note:|You Have Unread Mail. ][Wow!]");
					break;

				}

				case 4:		/* item 4 */
				{

					form_alert(1,"[1][Selected a meaningless |disabled Item!!!][Oh Boy!]");
					break;

				}

				case 5:		/* item 5 */
				{

					form_alert(1, "[1][You have selected |Quit.][Okay]");
					break;

				}

				default: break;

			}	/* end switch(item) */

			/* is item selected? */
			if(states[item] & PU_SELECTED)
			{

				/* we should deselect item now */
				states[item] = (short) (states[item] & ~PU_SELECTED);

				/* lets redisplay menu so they can see it */
				disp_popup(ENTRIES, entries, states);

			}	/* end if need to deselect item */

		}	/* end if button clicked */

		else		/* must be a keyboard event */
		{

			/* was return pressed */
			if((kpress & 0xFF) == 0x0D)	/* yes */
			{

				int index;			/* loop variable */


				/* find first default item */
				for(index = 0; index < ENTRIES; index++)
				{

					/* this item the default? */
					if(states[index] & PU_DEFAULT)	/* yes */
					{

						/* select this item */
						states[item] = (short) (states[item] | PU_SELECTED);

						/* display change */
						disp_popup(ENTRIES, entries, states);

						/* display a little alert */
						form_alert(1, "[1][You have selected |the default item|by <return>.][Okay!]");

						/* we should deselect item now */
						states[item] = (short) (states[item] & ~PU_SELECTED);

						/* display change */
						disp_popup(ENTRIES, entries, states);

						break;		/* exit for loop */

					}	/* end if default */

				}	/* end for */

			}	/* end if return pressed */

		}	/* end else MU_KEYBD message */

	}while(item != 5);

	/* close window */
	wind_close(windh);

	/* free wte_vt52.lib resources */
	wte_term(windh);

	/* delete window */
	wind_delete(windh);

	/* inform AES that we're exit app */
	appl_exit();

	/* return ok */
	return(0);

}	/* end of _main */

/*	disp_popup
 *
 */

void disp_popup(short numb_entries, char *entries[], short states[])
{

	int
		index;					/* for loop index value */

	short
		d;


	/* clear screen */
	wte_escf(windh, ESC_E, &d, &d);

	/* loop displaying each entry in turn */
	for(index = 0; index < numb_entries; index++)
	{

		/* set all attributes according to state */
		if(states[index] & PU_GRAYED)	/* disabled takes precedence */
		{

			/* set to grayed text */
			d = tGRAY;

		}

		else if(states[index] & PU_DEFAULT)
		{

			/* set to bold text */
			d = tBOLD;

		}
		else
		{

			/* else make sure its normal text */
			d = tNORMAL;

		}

		/* set new text */
		wte_escf(windh, ESC_F, &d, &d);	/* second d is ignored so I cheat */

		/* set color */
		if(states[index] & PU_RED)
		{

			/* set foreground (text) color to red */
			d = RED;

		}

		else
		{

			/* set foreground (text) color to black */
			d = BLACK;

		}

		/* set text color */
		wte_escf(windh, ESC_b, &d, &d);		/* second d is ignored so I cheat */

		/* check if selected */
		if(states[index] & PU_SELECTED)
		{

			/* turn reverse on */
			wte_escf(windh, ESC_p, &d, &d);	/* d's are ignored */

		}

		else
		{

			/* insure reverse off */
			wte_escf(windh, ESC_q, &d, &d);	/* d's are ignored */

		}

		/* Since I've insured that each entries text will fit into the window
		 * all we have to do is print the current entry string, we can skip
		 * all that nasty length testing, and clipping garabage :)
		 */

		wte_printf(windh, "%s\r", entries[index]);

		/* we want to avoid a last line scroll so we have to manual move
		 * the cursor down if needed (ie; every line except last!)
		 * The \r moves automatically moves the cursor back to the first
		 * column.
		 */

		if(index < (numb_entries - 1))
		{

			/* ok to move cursor down a line */
			wte_escf(windh, ESC_B, &d, &d);	/* d's are ignored */

		}

	}	/* end for */

}	/* end of disp_popup */
